@@ -1,10 +1,9 @@
 # coding: utf-8
 import os
 import webbrowser
-from typing import List, Callable
-from datetime import datetime
-import pandas as pd
 import traceback
+from typing import List, Callable
+from deprecated import deprecated
 from collections import OrderedDict
 from pyecharts.charts import Tab
 from pyecharts.components import Table
@@ -12,35 +11,8 @@ from pyecharts.options import ComponentTitleOpts
 
 from .utils.kline_generator import KlineGenerator
 from .enum import Mark, Direction, Operate, Freq
-from .objects import BI, FakeBI, FX, RawBar, NewBar, Event
+from .objects import BI, FakeBI, FX, RawBar, NewBar, Event, create_fake_bis
 from .utils.echarts_plot import kline_pro
-from .utils.ta import RSQ
-
-
-def create_fake_bis(fxs: List[FX]) -> List[FakeBI]:
-    """创建 fake_bis 列表
-
-    :param fxs: 分型序列，必须顶底分型交替
-    :return: fake_bis
-    """
-    if len(fxs) % 2 != 0:
-        fxs = fxs[:-1]
-
-    fake_bis = []
-    for i in range(1, len(fxs)):
-        fx1 = fxs[i-1]
-        fx2 = fxs[i]
-        assert fx1.mark != fx2.mark
-        if fx1.mark == Mark.D:
-            fake_bi = FakeBI(symbol=fx1.symbol, sdt=fx1.dt, edt=fx2.dt, direction=Direction.Up,
-                             high=fx2.high, low=fx1.low, power=round(fx2.high-fx1.low, 2))
-        elif fx1.mark == Mark.G:
-            fake_bi = FakeBI(symbol=fx1.symbol, sdt=fx1.dt, edt=fx2.dt, direction=Direction.Down,
-                             high=fx1.high, low=fx2.low, power=round(fx1.high-fx2.low, 2))
-        else:
-            raise ValueError
-        fake_bis.append(fake_bi)
-    return fake_bis
 
 
 def remove_include(k1: NewBar, k2: NewBar, k3: RawBar):
@@ -116,8 +88,13 @@ def check_fxs(bars: List[NewBar]) -> List[FX]:
     return fxs
 
 
-def check_bi(bars: List[NewBar]):
-    """输入一串无包含关系K线，查找其中的一笔"""
+def check_bi(bars: List[NewBar], bi_min_len: int = 7):
+    """输入一串无包含关系K线，查找其中的一笔
+
+    :param bars: 无包含关系K线列表
+    :param bi_min_len: 一笔的最少无包含关系K线数量，7是老笔的要求
+    :return:
+    """
     fxs = check_fxs(bars)
     if len(fxs) < 2:
         return None, bars
@@ -129,15 +106,18 @@ def check_bi(bars: List[NewBar]):
             fxs_b = [x for x in fxs if x.mark == Mark.G and x.dt > fx_a.dt and x.fx > fx_a.fx]
             if not fxs_b:
                 return None, bars
+
             fx_b = fxs_b[0]
             for fx in fxs_b:
                 if fx.high >= fx_b.high:
                     fx_b = fx
+
         elif fxs[0].mark == Mark.G:
             direction = Direction.Down
             fxs_b = [x for x in fxs if x.mark == Mark.D and x.dt > fx_a.dt and x.fx < fx_a.fx]
             if not fxs_b:
                 return None, bars
+
             fx_b = fxs_b[0]
             for fx in fxs_b[1:]:
                 if fx.low <= fx_b.low:
@@ -152,69 +132,24 @@ def check_bi(bars: List[NewBar]):
     bars_b = [x for x in bars if x.dt >= fx_b.elements[0].dt]
 
     # 判断fx_a和fx_b价格区间是否存在包含关系
-    ab_include = (fx_a.high > fx_b.high and fx_a.low < fx_b.low) or (fx_a.high < fx_b.high and fx_a.low > fx_b.low)
+    ab_include = (fx_a.high > fx_b.high and fx_a.low < fx_b.low) \
+                 or (fx_a.high < fx_b.high and fx_a.low > fx_b.low)
 
-    if len(bars_a) >= 7 and not ab_include:
-        # 计算笔的相关属性
-        power_price = round(abs(fx_b.fx - fx_a.fx), 2)
-        change = round((fx_b.fx - fx_a.fx) / fx_a.fx, 4)
+    if len(bars_a) >= bi_min_len and not ab_include:
         fxs_ = [x for x in fxs if fx_a.elements[0].dt <= x.dt <= fx_b.elements[2].dt]
-        fake_bis = create_fake_bis(fxs_)
-
-        bi = BI(symbol=fx_a.symbol, fx_a=fx_a, fx_b=fx_b, fxs=fxs_, fake_bis=fake_bis,
-                direction=direction, power=power_price, high=max(fx_a.high, fx_b.high),
-                low=min(fx_a.low, fx_b.low), bars=bars_a, length=len(bars_a),
-                rsq=RSQ([x.close for x in bars_a[1:-1]]), change=change)
-
+        bi = BI(symbol=fx_a.symbol, fx_a=fx_a, fx_b=fx_b, fxs=fxs_, direction=direction, bars=bars_a)
         return bi, bars_b
     else:
         return None, bars
 
 
-def get_sub_span(bis: List[BI], start_dt: [datetime, str], end_dt: [datetime, str], direction: Direction) -> List[BI]:
-    """获取子区间（这是进行多级别联立分析的关键步骤）
-
-    :param bis: 笔的列表
-    :param start_dt: 子区间开始时间
-    :param end_dt: 子区间结束时间
-    :param direction: 方向
-    :return: 子区间
-    """
-    start_dt = pd.to_datetime(start_dt)
-    end_dt = pd.to_datetime(end_dt)
-    sub = []
-    for bi in bis:
-        if bi.fx_b.dt > start_dt > bi.fx_a.dt:
-            sub.append(bi)
-        elif start_dt <= bi.fx_a.dt < bi.fx_b.dt <= end_dt:
-            sub.append(bi)
-        elif bi.fx_a.dt < end_dt < bi.fx_b.dt:
-            sub.append(bi)
-        else:
-            continue
-
-    if len(sub) > 0 and sub[0].direction != direction:
-        sub = sub[1:]
-    if len(sub) > 0 and sub[-1].direction != direction:
-        sub = sub[:-1]
-    return sub
-
-
-def get_sub_bis(bis: List[BI], bi: BI) -> List[BI]:
-    """获取大级别笔对象对应的小级别笔走势
-
-    :param bis: 小级别笔列表
-    :param bi: 大级别笔对象
-    :return:
-    """
-    sub_bis = get_sub_span(bis, start_dt=bi.fx_a.dt, end_dt=bi.fx_b.dt, direction=bi.direction)
-    if not sub_bis:
-        return []
-    return sub_bis
-
-
 class CZSC:
-    def __init__(self, bars: List[RawBar], max_bi_count=50, get_signals: Callable = None, verbose=False):
+    def __init__(self,
+                 bars: List[RawBar],
+                 max_bi_count: int = 50,
+                 bi_min_len: int = 7,
+                 get_signals: Callable = None,
+                 verbose=False):
         """
 
         :param bars: K线数据
@@ -225,6 +160,7 @@ class CZSC:
         """
         self.verbose = verbose
         self.max_bi_count = max_bi_count
+        self.bi_min_len = bi_min_len
         self.bars_raw = []  # 原始K线序列
         self.bars_ubi = []  # 未完成笔的无包含K线序列
         self.bi_list: List[BI] = []
@@ -272,32 +208,20 @@ class CZSC:
         max_high_ubi = max([x.high for x in bars_ubi[2:]])
 
         if last_bi.direction == Direction.Up and max_high_ubi > last_bi.high:
-            if min_low_ubi < last_bi.low and len(self.bi_list) > 2:
-                bars_ubi_a = self.bi_list[-2].bars \
-                             + [x for x in self.bi_list[-1].bars if x.dt > self.bi_list[-2].bars[-1].dt] \
-                             + [x for x in bars_ubi if x.dt > self.bi_list[-1].bars[-1].dt]
-                self.bi_list.pop(-1)
-                self.bi_list.pop(-1)
-            else:
-                bars_ubi_a = last_bi.bars + [x for x in bars_ubi if x.dt > last_bi.bars[-1].dt]
-                self.bi_list.pop(-1)
+            bars_ubi_a = last_bi.bars + [x for x in bars_ubi if x.dt > last_bi.bars[-1].dt]
+            self.bi_list.pop(-1)
+
         elif last_bi.direction == Direction.Down and min_low_ubi < last_bi.low:
-            if max_high_ubi > last_bi.high and len(self.bi_list) > 2:
-                bars_ubi_a = self.bi_list[-2].bars \
-                             + [x for x in self.bi_list[-1].bars if x.dt > self.bi_list[-2].bars[-1].dt] \
-                             + [x for x in bars_ubi if x.dt > self.bi_list[-1].bars[-1].dt]
-                self.bi_list.pop(-1)
-                self.bi_list.pop(-1)
-            else:
-                bars_ubi_a = last_bi.bars + [x for x in bars_ubi if x.dt > last_bi.bars[-1].dt]
-                self.bi_list.pop(-1)
+            bars_ubi_a = last_bi.bars + [x for x in bars_ubi if x.dt > last_bi.bars[-1].dt]
+            self.bi_list.pop(-1)
+
         else:
             bars_ubi_a = bars_ubi
 
         if self.verbose and len(bars_ubi_a) > 300:
             print(f"{self.symbol} - {self.freq} - {bars_ubi_a[-1].dt} 未完成笔延伸超长，延伸数量: {len(bars_ubi_a)}")
 
-        bi, bars_ubi_ = check_bi(bars_ubi_a)
+        bi, bars_ubi_ = check_bi(bars_ubi_a, self.bi_min_len)
         self.bars_ubi = bars_ubi_
         if isinstance(bi, BI):
             self.bi_list.append(bi)
@@ -355,9 +279,14 @@ class CZSC:
         if len(self.bi_list) > 0:
             bi = [{'dt': x.fx_a.dt, "bi": x.fx_a.fx} for x in self.bi_list] + \
                  [{'dt': self.bi_list[-1].fx_b.dt, "bi": self.bi_list[-1].fx_b.fx}]
+            fx = []
+            for bi_ in self.bi_list:
+                fx.extend([{'dt': x.dt, "fx": x.fx} for x in bi_.fxs[1:]])
         else:
             bi = None
-        chart = kline_pro(kline, bi=bi, width=width, height=height, title="{}-{}".format(self.symbol, self.freq.value))
+            fx = None
+        chart = kline_pro(kline, bi=bi, fx=fx, width=width, height=height,
+                          title="{}-{}".format(self.symbol, self.freq.value))
         return chart
 
     def open_in_browser(self, width: str = "1400px", height: str = '580px'):
@@ -373,22 +302,31 @@ class CZSC:
         chart.render(file_html)
         webbrowser.open(file_html)
 
+    @property
+    def last_bi_extend(self):
+        """判断最后一笔是否在延伸中，True 表示延伸中"""
+        if self.bi_list[-1].direction == Direction.Up \
+                and max([x.high for x in self.bars_ubi]) > self.bi_list[-1].high:
+            return True
+
+        if self.bi_list[-1].direction == Direction.Down \
+                and min([x.low for x in self.bars_ubi]) < self.bi_list[-1].low:
+            return True
+
+        return False
+
     @property
     def finished_bis(self) -> List[BI]:
         """返回当下基本确认完成的笔列表"""
         if not self.bi_list:
             return []
-
-        min_ubi = min([x.low for x in self.bars_ubi])
-        max_ubi = max([x.high for x in self.bars_ubi])
-        if (self.bi_list[-1].direction == Direction.Down and min_ubi >= self.bi_list[-1].low) \
-                or (self.bi_list[-1].direction == Direction.Up and max_ubi <= self.bi_list[-1].high):
-            bis = self.bi_list
         else:
-            bis = self.bi_list[:-1]
-        return bis
+            if self.last_bi_extend:
+                return self.bi_list[:-1]
+        return self.bi_list
 
 
+@deprecated(reason="这个版本不支持分批买卖，实现逻辑也过于复杂，放弃维护", version='1.0.0')
 class CzscTrader:
     """缠中说禅技术分析理论之多级别联立交易决策类"""
 
@@ -545,12 +483,6 @@ class CzscTrader:
         for freq, klines_ in klines_one.items():
             self.kas[freq].update(klines_[-1])
 
-        if self.op_freq:
-            opc: CZSC = self.kas[self.op_freq.value]
-            fbi = opc.finished_bis
-        else:
-            fbi = None
-
         self.symbol = self.kas["1分钟"].symbol
         self.end_dt = self.kas["1分钟"].bars_raw[-1].dt
         self.latest_price = self.kas["1分钟"].bars_raw[-1].close
@@ -586,32 +518,29 @@ class CzscTrader:
             assert self.cache['long_open_price'] > 0
             assert self.cache['long_max_high'] > 0
             assert self.cache['long_open_k1_id'] > 0
-            assert (self.op_freq and self.cache['long_open_error_price'] > 0) \
-                   or (not self.op_freq and self.cache['long_open_error_price'] == -1)
+            assert self.cache['long_open_error_price'] > 0
 
             if op['operate'] == Operate.LO.value:
                 op['operate'] = Operate.HL.value
-                self.cache['long_open_error_price'] = self._cal_open_error_price('long', max_open_tolerance)
-                # if fbi:
-                #     assert fbi[-1].direction == Direction.Down
-                #     max_low = max(fbi[-1].low, fbi[-3].low)
-                #     min_low = min(fbi[-1].low, fbi[-3].low)
-                #     self.cache['long_open_error_price'] = max(min_low, max_low * 0.97, self.latest_price * 0.95)
+                price_ = self._cal_open_error_price('long', max_open_tolerance)
+                if price_ > self.cache['long_open_error_price']:
+                    self.cache['long_open_error_price'] = price_
+
             else:
                 # 判断是否达到多头异常退出条件
                 if self.kg.m1[-1].id - self.cache['long_open_k1_id'] <= wait_time:
                     op['operate'] = Operate.LE.value
                     op['desc'] = f"long_wait_out"
 
-                if self.op_freq and self.latest_price < self.cache.get('long_open_error_price', 0):
+                if self.latest_price < self.cache['long_open_error_price']:
                     op['operate'] = Operate.LE.value
                     op['desc'] = f"long_open_error"
 
-                if self.latest_price < self.cache.get('long_max_high', 0) * (1 - stoploss):
+                if self.latest_price < self.cache['long_max_high'] * (1 - stoploss):
                     op['operate'] = Operate.LE.value
                     op['desc'] = f"long_stoploss_{stoploss}"
 
-                if self.kg.m1[-1].id - self.cache.get('long_open_k1_id', 99999999999) > timeout:
+                if self.kg.m1[-1].id - self.cache['long_open_k1_id'] > timeout:
                     op['operate'] = Operate.LE.value
                     op['desc'] = f"long_timeout_{timeout}"
 
@@ -619,12 +548,13 @@ class CzscTrader:
             assert self.cache['short_open_price'] > 0
             assert self.cache['short_min_low'] > 0
             assert self.cache['short_open_k1_id'] > 0
-            assert (self.op_freq and self.cache['short_open_error_price'] > 0) \
-                   or (not self.op_freq and self.cache['short_open_error_price'] == -1)
+            assert self.cache['short_open_error_price'] > 0
 
             if op['operate'] == Operate.SO.value:
                 op['operate'] = Operate.HS.value
-                self.cache['short_open_error_price'] = self._cal_open_error_price('short', max_open_tolerance)
+                price_ = self._cal_open_error_price('short', max_open_tolerance)
+                if price_ < self.cache['short_open_error_price']:
+                    self.cache['short_open_error_price'] = price_
 
             else:
                 # 判断是否达到空头异常退出条件
@@ -632,20 +562,21 @@ class CzscTrader:
                     op['operate'] = Operate.SE.value
                     op['desc'] = f"short_wait_out"
 
-                if self.op_freq and self.latest_price > self.cache['short_open_error_price'] > 0:
+                if self.latest_price > self.cache['short_open_error_price']:
                     op['operate'] = Operate.SE.value
                     op['desc'] = f"short_open_error"
 
-                if self.latest_price > self.cache.get('short_min_low', 10000000000) * (1 + stoploss):
+                if self.latest_price > self.cache['short_min_low'] * (1 + stoploss):
                     op['operate'] = Operate.SE.value
                     op['desc'] = f"short_stoploss_{stoploss}"
 
-                if self.kg.m1[-1].id - self.cache.get('short_open_k1_id', 99999999999) > timeout:
+                if self.kg.m1[-1].id - self.cache['short_open_k1_id'] > timeout:
                     op['operate'] = Operate.SE.value
                     op['desc'] = f"short_timeout_{timeout}"
 
         else:
             assert last_op == Operate.HO.value
+            # HO 状态下的 LE、SE 是无效的
             if op['operate'] in [Operate.LE.value, Operate.SE.value]:
                 op['operate'] = Operate.HO.value
 
@@ -673,8 +604,7 @@ class CzscTrader:
             assert self.cache['long_open_k1_id'] > 0
             self.cache['long_max_high'] = max(self.latest_price, self.cache['long_max_high'])
             assert self.cache['long_max_high'] > 0
-            assert (self.op_freq and self.cache['long_open_error_price'] > 0) \
-                   or (not self.op_freq and self.cache['long_open_error_price'] == -1)
+            assert self.cache['long_open_error_price'] > 0
 
         elif op['operate'] == Operate.SE.value:
             self.cache.update({
@@ -699,8 +629,7 @@ class CzscTrader:
             assert self.cache['short_open_k1_id'] > 0
             self.cache['short_min_low'] = min(self.latest_price, self.cache['short_min_low'])
             assert self.cache['short_min_low'] > 0
-            assert (self.op_freq and self.cache['short_open_error_price'] > 0) \
-                   or (not self.op_freq and self.cache['short_open_error_price'] == -1)
+            assert self.cache['short_open_error_price'] > 0
 
         else:
             assert op['operate'] == Operate.HO.value
