@@ -1,7 +1,9 @@
 # coding: utf-8
 
 from .echarts_plot import kline_pro, heat_map
-from .kline_generator import KlineGenerator
+from .kline_generator import KlineGenerator, KlineGeneratorD
+from .bar_generator import BarGenerator
 from .ta import KDJ, MACD, EMA, SMA
 from .io import read_pkl, save_pkl, read_json, save_json
 from .log import create_logger
+from .word_writer import WordWriter
