@@ -1,10 +1,10 @@
 # coding: utf-8
 import zipfile
 from tqdm import tqdm
+import pandas as pd
 from czsc.analyze import *
 from czsc.enum import Freq, Operate
-from czsc import signals
-from czsc.signals import get_default_signals, get_s_three_bi
+from czsc.signals.signals import get_default_signals, get_s_three_bi, get_s_d0_bi
 from czsc.objects import Event, Factor, Signal
 
 cur_path = os.path.split(os.path.realpath(__file__))[0]
@@ -26,14 +26,18 @@ def read_1min():
     return bars
 
 
-def test_find_bi():
+def read_daily():
     file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
     kline = pd.read_csv(file_kline, encoding="utf-8")
     kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
-    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.F1, open=row['open'], dt=row['dt'],
+    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
                    close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
             for i, row in kline.iterrows()]
+    return bars
 
+
+def test_find_bi():
+    bars = read_daily()
     # 去除包含关系
     bars1 = []
     for bar in bars:
@@ -76,21 +80,20 @@ def get_user_signals(c: CZSC) -> OrderedDict:
 
 
 def test_czsc_update():
-    file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
-    kline = pd.read_csv(file_kline, encoding="utf-8")
-    kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
-    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
-                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
-            for i, row in kline.iterrows()]
-
+    bars = read_daily()
     # 不计算任何信号
     c = CZSC(bars, max_bi_count=50)
     assert not c.signals
 
     # 计算信号
-    c = CZSC(bars, max_bi_count=50, get_signals=get_default_signals)
+    c = CZSC(bars, max_bi_count=50, get_signals=get_default_signals, verbose=True)
+    assert len(c.bi_list) == 50 and not c.last_bi_extend
     assert isinstance(c.signals, OrderedDict) and len(c.signals) == 38
 
+    last_bi = c.bi_list[-1]
+    assert len(last_bi.raw_bars) == 32 and last_bi.power_price == last_bi.power
+    assert len(last_bi.fake_bis) == 11
+    assert last_bi.fake_bis[0].direction == last_bi.fake_bis[-1].direction == last_bi.direction
     # 测试自定义信号
     c = CZSC(bars, max_bi_count=50, get_signals=get_user_signals)
     assert len(c.signals) == 10
@@ -104,7 +107,7 @@ def test_czsc_update():
     os.remove(file_html)
 
 
-def test_czsc_trader():
+def drop_test_czsc_trader():
     bars = read_1min()
     kg = KlineGenerator(max_count=3000, freqs=['1分钟', '5分钟', '15分钟', '30分钟', '60分钟', '日线'])
     for row in tqdm(bars[:-10000], desc='init kg'):
@@ -123,28 +126,22 @@ def test_czsc_trader():
         ]),
     ]
     ct = CzscTrader(op_freq=Freq.F5, kg=kg, get_signals=get_default_signals, events=events)
-    assert len(ct.s) == 214
+    assert len(ct.s) == 215
     for row in bars[-10000:]:
         op = ct.check_operate(row)
         print(" : op    : ", op)
         print(" : cache : ", dict(ct.cache), "\n")
-    assert len(ct.s) == 214
+    assert len(ct.s) == 215
 
 
 def test_get_signals():
 
     def get_test_signals(c: CZSC) -> OrderedDict:
         s = OrderedDict({"symbol": c.symbol, "dt": c.bars_raw[-1].dt, "close": c.bars_raw[-1].close})
-        s.update(signals.get_s_d0_bi(c))
+        s.update(get_s_d0_bi(c))
         return s
 
-    file_kline = os.path.join(cur_path, "data/000001.SH_D.csv")
-    kline = pd.read_csv(file_kline, encoding="utf-8")
-    kline.loc[:, "dt"] = pd.to_datetime(kline.dt)
-    bars = [RawBar(symbol=row['symbol'], id=i, freq=Freq.D, open=row['open'], dt=row['dt'],
-                   close=row['close'], high=row['high'], low=row['low'], vol=row['vol'])
-            for i, row in kline.iterrows()]
-
+    bars = read_daily()
     # 不计算任何信号
     c = CZSC(bars, max_bi_count=50, get_signals=get_test_signals)
     assert c.signals['日线_倒0笔_方向'] == '向下_任意_任意_0'
